#define RELAY_PIN  5 

//http://www.electroschematics.com/wp-content/uploads/2015/02/DHT.rar
#include "DHT.h" 
#define DHT_PIN  6 //A4//6     // what pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302)
DHT dht(DHT_PIN, DHTTYPE);

#define LATCH_PIN 4
#define CLK_PIN 7
#define DATA_PIN 8
 
#define BUZZER_PIN 3
 
#include <TimerOne.h>
 
#include <SanguruSegment.h>
SanguruSegment segment;
 
#include <SanguruButton.h>
SanguruButton btn0(A1);
SanguruButton btn1(A2);
SanguruButton btn2(A3);
// btnx.getButton()값 
#define ON_RELEASED 1
#define ON_PRESSED 2
#define RELEASED 4
#define PRESSED 8
#define REPEAT 16
#define LONG_PRESSED 32
 
#define NO_OF_MENU 4 
#define MENU_MAIN 0
#define MENU_SWITCH 1
#define MENU_COUNT_DOWN 2
#define MENU_TIME 3
// 숫자가 변경된 후 이 시간 이내에는 깜빡거리지 않는다 
#define NO_BLINK_MIL 1000
 
char *menuTitle[NO_OF_MENU] = {"main","swch","cntd","time"};
 
byte thisMenu = 0;
// 메인메뉴 타이틀 표시하기 위하여(thisMenu != prevMenu) 0이 아니라 -1로 지정 
byte prevMenu = -1;
byte menuVal,upVal,downVal;

#define DUST_PWR_PIN 9
#define DUST_PIN A5

unsigned long thisMil;
unsigned long nextSecMil;
unsigned long thisSec;
int loopCnt;

// variables for dust senser
int samplingMicroSec = 280;
int deltaMicroSec = 40;
int sleepMicroSec = 9680;
float dustVal = 0;
float smoothDustVal = 0.8;
float calcVoltage;
float dustDensity = 0;

// variables for DHT22
float temperature;
float humidity;
int  relayOnOff = 0; // 0:off 1:on

// time
int dd,hh,mm,ss;

// begin class SanguruTime
#define ONE_DAY_MIL 86400000L
class SanguruTime {
	public:
	SanguruTime();
	void getTime (int *HH, int *MM, int *SS);
	void setTime (int HH, int MM, int SS);
	private:
	unsigned long timeDiffrenceMil;
};

SanguruTime::SanguruTime() {
}

void SanguruTime::getTime(int *HH, int *MM, int *SS) {
	
	unsigned long timeMil = (millis() + timeDiffrenceMil) % ONE_DAY_MIL;
	unsigned long timeSec;

	timeSec = timeMil / 1000L; // time sec
	
	*HH = timeSec / 3600;
	*MM = (timeSec / 60) % 60;
	*SS = timeSec % 60;

}

void SanguruTime::setTime(int HH, int MM, int SS) {
	#define ONE_DAY_MIL 86400000L
	unsigned long  hh = HH;
	unsigned long  mm = MM;
	unsigned long  ss = SS;

    unsigned long thisMil = millis();
	unsigned long timeMil = (thisMil / ONE_DAY_MIL + 1L) * ONE_DAY_MIL  + (hh * 3600L + mm * 60L + ss ) * 1000L;
	
	timeDiffrenceMil = (timeMil - thisMil) % ONE_DAY_MIL;
}

SanguruTime t;
 
void setup() {
  // 값은 250~1000 사이가 적당함 
  int intervalMicroSec = 1000;
  Timer1.initialize(intervalMicroSec);
  // 백그라운드에서 반복적으로 수행할 ISR을 지정한다.
  // ISR에는 세그먼트와 버튼 등의 update()메쏘드를 지정하면 된다.
  Timer1.attachInterrupt(ISRupdate);
  // 숫자 표시가 역순이면 LOW와 HIGH를 서로 바꾸면 된다  
  byte digitStartFrom = LOW;
  segment.init(LATCH_PIN,CLK_PIN,DATA_PIN,digitStartFrom,intervalMicroSec);
  // setLongPress(true)와 setRepeat(true)는 둘중 하나만 지정이 유효함
  btn0.setLongPress(true);  // menu
  btn1.setRepeat(true);   // up
  btn2.setRepeat(true);   // down

  pinMode(RELAY_PIN,OUTPUT);
  digitalWrite(RELAY_PIN,HIGH); // relay off

  pinMode(BUZZER_PIN,OUTPUT);
  // PNP트란지스터에 연결되어 있으므로 LOW일 때 전류가 흘러 부저가 울림 
  // 소리가 나지 않도록 즉시 조처 
  digitalWrite(BUZZER_PIN,HIGH);

  pinMode(DUST_PWR_PIN,OUTPUT);
  digitalWrite(DUST_PWR_PIN,HIGH); // turn the LED off

  dht.begin();  
 
  Serial.begin(9600);
  hh = 12;
  mm = 0;
  ss = 0;
  t.setTime(hh,mm,ss);
}

void loop() {
  // 외부 인터럽트를 먼저 처리한다 
  menuVal = btn0.getButton();
  upVal = btn1.getButton();
  downVal = btn2.getButton();

  while (Serial.available()) {
  	readSerialAndSetOptions();
  }

  // 메뉴별로 로직을 구분한다 
  switch (thisMenu) {
    case MENU_MAIN       : doMain();     break;
    case MENU_SWITCH     : doSwitch();   break;
    case MENU_COUNT_DOWN : doCountDown();break;
    case MENU_TIME       : doTime();     break;
  }
  
}

void readSerialAndSetOptions() {
  #define READ_STR_LEN 20
  #define KEY_STR_LEN 10
  #define VALUE_STR_LEN 20
  static char readStr[READ_STR_LEN];
  static int readIdx = 0;
  char c = Serial.read();

  if (c == '\r' || c == '\n')
    return;

  readStr[readIdx] = c;
  readStr[++readIdx] = '\0';
  if (readIdx >= READ_STR_LEN)
  	readIdx = 0;

  if (c == '|') {
  	char key[KEY_STR_LEN];
  	char value[VALUE_STR_LEN];
  	char *ptr;

  	ptr = strtok(readStr,",");
  	strncpy(key,ptr,KEY_STR_LEN);

  	ptr = strtok(NULL,"|");
  	strncpy(value,ptr,VALUE_STR_LEN);

  	setOptions(key,value);

  	readIdx = 0;
  }
}

void setOptions(char *key, char *value) {
  // int i = atoi(value);
  // float f = atof(value);

  #define STRCMP_EQUAL 0
  #define RELAY_ON 0
  #define RELAY_OFF 1

  
  if (strcmp(key,"r") == STRCMP_EQUAL) {
  	if (strcmp(value,"1") == STRCMP_EQUAL) {
  	  relayOnOff = 1;
  	  digitalWrite(RELAY_PIN,RELAY_ON);
  	}
  	else {
  	  relayOnOff = 0;
  	  digitalWrite(RELAY_PIN,RELAY_OFF);
    }
  	printToSerial('r',relayOnOff);
  }
  else  if (strcmp(key,"h") == STRCMP_EQUAL) {
  	hh = atoi(value);
  	t.setTime(hh,mm,ss);
  }
  else  if (strcmp(key,"m") == STRCMP_EQUAL) {
  	mm = atoi(value);
  	t.setTime(hh,mm,ss);
  }
  else  if (strcmp(key,"s") == STRCMP_EQUAL) {
  	ss = atoi(value);
  	t.setTime(hh,mm,ss);
  }

}

void printToSerial(char c, int v) {
  Serial.print(c);
  Serial.print(',');
  Serial.print(v);
  Serial.print('|');
}

void printToSerial(char c, float v, int decimal) {
  Serial.print(c);
  Serial.print(',');
  Serial.print(v,decimal);
  Serial.print('|');
}
// ISR에는 파라메터를 지정할 수 없다
//여기서는 세그먼트와 버튼의 update()메쏘드를 지정한다
void ISRupdate() {
  segment.update();
  btn0.update();
  btn1.update();
  btn2.update();
}

void doMain() {
  // 세그먼트에 표시할 값은 변경된 경우에만 write()한다
  // 만약 쉼없이 표시하면 숫자가 깜빡거린다
  // 이경우 loop()안에 약간의 delay를(예30) 주면 깜빡거림이 없어진다.
  if(prevMenu!= thisMenu) {
    segment.write(menuTitle[MENU_MAIN]);
    delay(1000);
    prevMenu= thisMenu;
  }

  if (nextMenu())
  	return;
 
  thisMil = millis();
  if (thisMil < nextSecMil)
    return;
  else 
  	nextSecMil = thisMil + 1000;
    
  loopCnt++;
  if (loopCnt > 14 )
    loopCnt = 0;

  if (loopCnt == 0) {
  	segment.clear();
  	delay(100);
    Timer1.stop();
    
    for (int i = 0; i < 100; i++) {
    
      digitalWrite(DUST_PWR_PIN,LOW); // power on the LED
      delayMicroseconds(samplingMicroSec);
 
      dustVal = analogRead(DUST_PIN); // read the dust value via pin 5 on the sensor
      delayMicroseconds(deltaMicroSec);
 
      digitalWrite(DUST_PWR_PIN,HIGH); // turn the LED off
      delayMicroseconds(sleepMicroSec);
      smoothDustVal = dustVal * 0.005 + smoothDustVal * 0.995; 
       
    }  
  
    calcVoltage = smoothDustVal * (5.0/1023.0);
    //dustDensity = 0.17*(dustVal*0.0049)-0.1;
    dustDensity = 0.17 * calcVoltage - 0.1;

    Timer1.resume(); 
    printToSerial('d', (int)(dustDensity*1000));
  }
  else if (loopCnt == 4 ) {
  	temperature = dht.readTemperature();
    humidity = dht.readHumidity();
    if (isnan(temperature) || isnan(humidity)) {
      //Serial.println("e 1");
      //Serial.println("Failed to read from DHT sensor!");
      temperature = 0;
      humidity = 0;
    }
    printToSerial('t',temperature,1);
    printToSerial('h',humidity,1);
    
  }
  else if (loopCnt == 6 ) {
  	 printToSerial('r',relayOnOff);
  }

  if (loopCnt == 0 ) {
    segment.setWindow(0,3);
    segment.write((int)(dustDensity*1000));
    segment.setWindow(3,1);
    segment.write("u");
  }
  else if (loopCnt == 4 ) {
    segment.setWindow(0,3);
    segment.write(temperature,1);
    segment.setWindow(3,1);
    segment.write("c");    
  }
  else if (loopCnt == 8 ) {
    segment.setWindow(0,3);
    segment.write(humidity,0);
    segment.setWindow(3,1);
    segment.write("h");    
  }
  else if (loopCnt == 12 ) {
  	segment.setWindow(0,4);
    t.getTime(&hh,&mm,&ss);
    float HHMM = hh + (float)mm / 100.0;
    segment.write(HHMM,2,'0');  // preceding '0'
    segment.blinkDot(true);
  }

}


void doSwitch() {
  if(prevMenu!= thisMenu) {
    segment.write(menuTitle[MENU_SWITCH]);
    prevMenu= thisMenu;
  }
  nextMenu();
}
 
void doCountDown() {
  #define MODE_DISPLAY 0
  #define MODE_EDIT 1
  #define MODE_ON_COUNT_DOWN 2
 
  static byte mode = MODE_DISPLAY; 
 
  static int countDownVal = 0;
  static int prevCountDownVal = 0;
  static int SVcountDownVal = 0;
  static unsigned long nextCountDownMil;
  static unsigned long upDownButtonPressedMil;
 
  // 처음 들어 올 때 
  if (prevMenu != thisMenu) {
  	segment.blink(false);
  	// 타이틀 표시
  	segment.write(menuTitle[MENU_COUNT_DOWN]);   
    prevCountDownVal = countDownVal;
    // 숫자 표시
    delay(1000); 
    segment.write(countDownVal);
    prevMenu = thisMenu;
  }
  // 표시 모드
  else if (mode == MODE_DISPLAY) {
    // 다음 메뉴로 이동
    if (menuVal == RELEASED) {
      segment.blink(false);
      nextMenu();
    } 
    // 편집 모드 진입,깜빡거림 시작    
    else if (menuVal == LONG_PRESSED) {
      mode = MODE_EDIT;
      // 타이틀이 표시되어 있는 경우 0으로 바꿔 표시하기 위함
      segment.write(countDownVal);   
      segment.blink(true);
      // 숫자가 변경된 직후에는 깜빡거리지 않게 하기 위하여 지정 
      upDownButtonPressedMil = millis();
    }
    // 카운트 다운 모드 진입
    else if ((upVal == RELEASED || downVal == RELEASED) && countDownVal > 0 ) {
      mode = MODE_ON_COUNT_DOWN;
      SVcountDownVal = countDownVal;
      nextCountDownMil = millis() + 1000; 
    }
  }
  // 편집 모드
  else if (mode == MODE_EDIT) {
    // 숫자가 변경된 직후에는 깜빡거리지 않게 한다
    if (millis() - upDownButtonPressedMil <  NO_BLINK_MIL )
      segment.blink(false);
    else 
      segment.blink(true);
    // 편집 모드 해제,깜빡거림 중지 
    if (menuVal == RELEASED) {
      mode = MODE_DISPLAY;
      segment.blink(false);
    }
    // UP
    else if (upVal == RELEASED || upVal == REPEAT) {
      countDownVal +=10;
      // 숫자가 변경된 직후에는 깜빡거리지 않게 하기 위하여 지정
      upDownButtonPressedMil = millis();
    }
    // DOWN  
    else if (downVal == RELEASED || downVal == REPEAT) {
      countDownVal -=10;
      // 숫자가 변경된 직후에는 깜빡거리지 않게 하기 위하여 지정 
      upDownButtonPressedMil = millis();     
    }
 
    if (countDownVal < 0 )
      countDownVal = 0;
    else if (countDownVal > 9999 )
      countDownVal = 9999;
  }  
  // 카운트 다운 모드
  else if (mode == MODE_ON_COUNT_DOWN) {
    // 카운트 다운 중지하고 이전 세팅 값 표시
    if (upVal == RELEASED || downVal == RELEASED) {
      mode = MODE_DISPLAY;
      countDownVal = SVcountDownVal; 
    }
    // 1초가 지나면 1씩 다운 
    else if (millis() > nextCountDownMil) {
      nextCountDownMil = millis() + 1000; 
      countDownVal -= 1;
      
      // 0이 되면 부저 울림
      if (countDownVal == 0) {
        digitalWrite(BUZZER_PIN,LOW);
        delay(500);
        digitalWrite(BUZZER_PIN,HIGH);
      }
      if (countDownVal < -999)  
        countDownVal = -999;
    }
  } 
  
  // 값이 바뀌면 다시 표시
  if (prevCountDownVal != countDownVal) {
    prevCountDownVal = countDownVal;
    segment.write(countDownVal);
  }
}
 
void doTime() {

  if(prevMenu!= thisMenu) {
    segment.write(menuTitle[MENU_TIME]);
    delay(1000);
    prevMenu= thisMenu;
  }

  if (nextMenu())
  	return;

  thisMil = millis();
  if (thisMil < nextSecMil)
    return;
  else 
  	nextSecMil = thisMil + 1000;
    
  t.getTime(&hh,&mm,&ss);
  float HHMM = hh + (float)mm / 100.0;
  segment.write(HHMM,2,'0');  // preceding '0'
  segment.blinkDot(true);
}
 
bool nextMenu() {
  bool returnVal;
  if (menuVal == RELEASED) {
    prevMenu = thisMenu;
    thisMenu++;
    if (thisMenu >= NO_OF_MENU)
    thisMenu = 0;
    returnVal = true;
    segment.clear();    
  }
  else {
    returnVal = false;
  }

  return returnVal;
}